# Aura Engine

[![Contributor Covenant](https://img.shields.io/badge/Contributor%20Covenant-2.1-4baaaa.svg)](https://docs.aura.radio/en/latest/contribute/code_of_conduct.html) [![Latest Release](https://gitlab.servus.at/aura/engine/-/badges/release.svg)](https://gitlab.servus.at/aura/engine/-/releases) [![pipeline status](https://gitlab.servus.at/aura/engine/badges/main/pipeline.svg)](https://gitlab.servus.at/aura/engine/-/commits/main) [![coverage report](https://gitlab.servus.at/aura/engine/badges/main/coverage.svg?job=run_test_cases)](https://gitlab.servus.at/aura/engine/-/commits/main) [![security: bandit](https://img.shields.io/badge/security-bandit-yellow.svg)](https://gitlab.servus.at/aura/engine/-/commits/main) [![Code style: black](https://img.shields.io/badge/code%20style-black-000000.svg)](https://github.com/psf/black)

<img src="https://gitlab.servus.at/aura/aura/-/raw/main/assets/images/aura-engine.png" width="120" align="right" />

AURA Engine is a scheduling service to control the playout of _Engine Core_.

This documentation is primarily meant for developers. For using the AURA Community Radio Suite
check out the documentation at [docs.aura.radio](https://docs.aura.radio/)

To learn more about Automated Radio go to [aura.radio](https://aura.radio).

## Overview

The following diagram gives an simplified overview on the architecture.

```mermaid
flowchart LR

    api-- GET calendar -->steering[Steering API]
    api-- GET playlist -->tank[Tank API]
    ea[Engine API]
    events-- on_boot\non_sick\non_resurrect -->monitor[Monitor]
    events-- on_play\non_fallback_active -->clock[Clock]
    clock-- POST clock_data -->ea
    monitor-- PING heartbeat -->monserver[Monitoring Server]
    monitor-- POST health_data -->ea
    mixer-- UNIX SOCKET -->engine_core

    subgraph external
        ea
        monserver
        engine_core
        steering
        tank
    end
    subgraph engine
        engine_instance[Engine]-.-oplayer & events & scheduler

        scheduler-- control\ncommand\ntimer -->player[Player]

        player-- control action -->mixer
        events[Event Dispatcher *]

        subgraph scheduling
            api[Fetch & Cache API]
            timetable<-->api
            scheduler<-->timetable[Timetable Management]
            scheduler-- cycle -->scheduler
        end
        subgraph core[core]
            mixer[Mixer]
        end
        subgraph plugins
            clock
            monitor
        end
    end

    style external fill:#FFF
    style ea fill:#FFF,stroke:#333,stroke-width:1px
    style steering fill:#FFF,stroke:#333,stroke-width:1px
    style tank fill:#FFF,stroke:#333,stroke-width:1px
    style monserver fill:#FFF,stroke:#333,stroke-width:1px
    style engine_core fill:#FFF,stroke:#333,stroke-width:1px
```

(\*) **Event Dispatcher**: Events are issued and consumed in multiple locations of Engine. The diagram shows only a few ones, in order to support the overview. Similar to other artifacts, which are not part of this diagram.

## Prerequisites

Before you begin, ensure you have met the following requirements:

- Operating system: Debian 12, Ubuntu 23.10 or newer
- `git`, `make`
- [Docker](https://www.docker.com/), optional if you want to run in a container
- [Python 3.11+](https://www.python.org/downloads/)
- [Poetry](https://python-poetry.org/)

Ensure that you have also dependencies such as `steering`, `tank`, `dashboard`, `engine-core`, and `engine-api` up and running.

## Preparation

### Initialize environment

Install dependencies and prepare config file:

```shell
make init.app
```

This also creates a default configuration file at `config/engine.yaml`.

For development install with:

```shell
make init.dev
```

Note, if some configuration exists under `/etc/aura/engine.yaml` the configuration by default is drawn from there. This overrides any configuration located in the local configuration file.

## Configuration

Edit the configuration file `config/engine.yaml`. Verify or change at least these config options:

```yaml
# The secret which is used to authenticate against Tank
api_tank_secret: aura-engine-secret
```

## Running Engine

To start the Engine execute:

```shell
make run
```

## Docker

For production deployments follow the Docker Compose installation instruction for _AURA Playout_ at [docs.aura.radio](https://docs.aura.radio/).

The following instructions are meant for development.

### Build with Docker

Build your own, local Docker image

```shell
make docker.build
```

### Run with Docker

Run the locally build image

```shell
make docker.run
```

### Release to DockerHub

Releasing a new version to DockerHub

```shell
make docker.push
```

Usually this is not required, as it is done automatically by the CI/CD pipeline.

## Read more

- [Engine Developer Guide](docs/developer-guide.md)
- [Setting up the Audio Store](https://docs.aura.radio/en/latest/administration/setup-audio-store.html)
- [docs.aura.radio](https://docs.aura.radio)
- [aura.radio](https://aura.radio)
