import datetime
from typing import TYPE_CHECKING, Any, Dict, List, Optional, Type, TypeVar

import attr
from dateutil.parser import isoparse

if TYPE_CHECKING:
    from ..models.playout_episode import PlayoutEpisode
    from ..models.playout_schedule import PlayoutSchedule
    from ..models.playout_show import PlayoutShow
    from ..models.time_slot import TimeSlot


T = TypeVar("T", bound="PlayoutProgramEntry")


@attr.s(auto_attribs=True)
class PlayoutProgramEntry:
    """
    Attributes:
        id (str):
        start (datetime.datetime):
        end (datetime.datetime):
        show_id (int):
        show (PlayoutShow):
        timeslot_id (Optional[int]):
        playlist_id (Optional[int]):
        timeslot (Optional[TimeSlot]):
        episode (Optional[PlayoutEpisode]):
        schedule (Optional[PlayoutSchedule]):
    """

    id: str
    start: datetime.datetime
    end: datetime.datetime
    show_id: int
    show: "PlayoutShow"
    timeslot_id: Optional[int]
    playlist_id: Optional[int]
    timeslot: Optional["TimeSlot"]
    episode: Optional["PlayoutEpisode"]
    schedule: Optional["PlayoutSchedule"]
    additional_properties: Dict[str, Any] = attr.ib(init=False, factory=dict)

    def to_dict(self) -> Dict[str, Any]:
        id = self.id
        start = self.start.isoformat()

        end = self.end.isoformat()

        show_id = self.show_id
        show = self.show.to_dict()

        timeslot_id = self.timeslot_id
        playlist_id = self.playlist_id
        timeslot = self.timeslot.to_dict() if self.timeslot else None

        episode = self.episode.to_dict() if self.episode else None

        schedule = self.schedule.to_dict() if self.schedule else None

        field_dict: Dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update(
            {
                "id": id,
                "start": start,
                "end": end,
                "showId": show_id,
                "show": show,
                "timeslotId": timeslot_id,
                "playlistId": playlist_id,
                "timeslot": timeslot,
                "episode": episode,
                "schedule": schedule,
            }
        )

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        from ..models.playout_episode import PlayoutEpisode
        from ..models.playout_schedule import PlayoutSchedule
        from ..models.playout_show import PlayoutShow
        from ..models.time_slot import TimeSlot

        d = src_dict.copy()
        id = d.pop("id")

        start = isoparse(d.pop("start"))

        end = isoparse(d.pop("end"))

        show_id = d.pop("showId")

        show = PlayoutShow.from_dict(d.pop("show"))

        timeslot_id = d.pop("timeslotId")

        playlist_id = d.pop("playlistId")

        _timeslot = d.pop("timeslot")
        timeslot: Optional[TimeSlot]
        if _timeslot is None:
            timeslot = None
        else:
            timeslot = TimeSlot.from_dict(_timeslot)

        _episode = d.pop("episode")
        episode: Optional[PlayoutEpisode]
        if _episode is None:
            episode = None
        else:
            episode = PlayoutEpisode.from_dict(_episode)

        _schedule = d.pop("schedule")
        schedule: Optional[PlayoutSchedule]
        if _schedule is None:
            schedule = None
        else:
            schedule = PlayoutSchedule.from_dict(_schedule)

        playout_program_entry = cls(
            id=id,
            start=start,
            end=end,
            show_id=show_id,
            show=show,
            timeslot_id=timeslot_id,
            playlist_id=playlist_id,
            timeslot=timeslot,
            episode=episode,
            schedule=schedule,
        )

        playout_program_entry.additional_properties = d
        return playout_program_entry

    @property
    def additional_keys(self) -> List[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties
