#
# Aura Engine (https://gitlab.servus.at/aura/engine)
#
# Copyright (C) 2017-now() - The Aura Engine Team.

# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU Affero General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.

# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Affero General Public License for more details.

# You should have received a copy of the GNU Affero General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.


import pathlib
import unittest

from aura_engine.base.config import AuraConfig
from aura_engine.base.lang import DotDict
from aura_engine.base.utils import SimpleUtil as SU
from aura_engine.plugins.clock import ClockInfoHandler
from aura_engine.scheduling.domain import Playlist, PlaylistItem, Timeslot
from aura_engine.scheduling.timetable import TimetableService


class TestClockHandler(unittest.TestCase):
    """
    Testing the timetable service.
    """

    config = None
    engine = None

    #
    # Setup and teardown
    #

    def setUp(self):
        self.config = AuraConfig.instance.config
        cache_location = self.config.general.cache_dir
        self.timetable = TimetableService(cache_location)
        f = self.timetable.timetable_file
        self.timetable.timetable_file = f.replace("timetable.json", "timetable-test.json")
        self.engine = DotDict({"scheduler": DotDict({"timetable": self.timetable})})

    def tearDown(self):
        pass

    #
    # Test Cases
    #

    def test_clock_scheduler_cycle(self):
        print(self._testMethodName)

        clockHandler = ClockInfoHandler(self.engine)

        # Not yet implemented, so it passes in any case
        timetable = None
        clockHandler.on_scheduler_cycle(timetable)

    def test_clock_on_fallback_active(self):
        print(self._testMethodName)

        now = SU.timestamp()
        ts1 = Timeslot(
            id=1, repetition_id=None, start=now - 500, end=now - 400, show=None, episode=None
        )
        ts2 = Timeslot(
            id=1, repetition_id=None, start=now - 400, end=now + 400, show=None, episode=None
        )
        ts3 = Timeslot(
            id=1, repetition_id=None, start=now + 400, end=now + 540, show=None, episode=None
        )
        ts4 = Timeslot(
            id=1, repetition_id=None, start=now + 540, end=now + 720, show=None, episode=None
        )

        pl = Playlist("999", "Playlist X", False)
        alpha = PlaylistItem("file://alpha.flac", 100, 100, None)
        beta = PlaylistItem("file://beta.flac", 600, 100, None)
        gamma = PlaylistItem("file://gamma.flac", 100, 100, None)
        pl.add(alpha)
        pl.add(beta)
        pl.add(gamma)

        ts2.set_playlists(pl, None, None)
        self.timetable.timetable = [ts1, ts2, ts3, ts4]

        clockHandler = ClockInfoHandler(self.engine)
        clockHandler.on_fallback_active(ts1)

    def test_clock_on_play(self):
        print(self._testMethodName)

        now = SU.timestamp()
        ts1 = Timeslot(
            id=1, repetition_id=None, start=now - 500, end=now - 400, show=None, episode=None
        )
        ts2 = Timeslot(
            id=1, repetition_id=None, start=now - 400, end=now + 400, show=None, episode=None
        )
        ts3 = Timeslot(
            id=1, repetition_id=None, start=now + 400, end=now + 540, show=None, episode=None
        )
        ts4 = Timeslot(
            id=1, repetition_id=None, start=now + 540, end=now + 720, show=None, episode=None
        )

        pl = Playlist("999", "Playlist X", False)
        alpha = PlaylistItem("file://alpha.flac", 100, 100, None)
        beta = PlaylistItem("file://beta.flac", 600, 100, None)
        gamma = PlaylistItem("file://gamma.flac", 100, 100, None)
        pl.add(alpha)
        pl.add(beta)
        pl.add(gamma)

        ts2.set_playlists(pl, None, None)
        self.timetable.timetable = [ts1, ts2, ts3, ts4]

        clockHandler = ClockInfoHandler(self.engine)
        clockHandler.on_play(beta)


if __name__ == "__main__":
    unittest.main()
